@@ -28,7 +28,6 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include "loader.h"
 #include "vmelilo.h"
 
 /*--------------------------------------------------------------------------*/
@@ -454,7 +453,10 @@ create_map_data
 	const char			*kernel;
 	const char			*password;
 	const char			*cmdline;
-	char				*p;
+	const char			*append;
+	const char			*console;
+	const char			*root;
+	const u_long		*read_only;
 	const u_long		*restricted;
 
 #define WRITE_TAG_STRING(f, tag, arg) \
@@ -472,6 +474,8 @@ create_map_data
 	write_tag_data  (&map_data, TAG_HEADER,     NULL,                        0);
 	WRITE_TAG_STRING(&map_data, TAG_DEFAULT,    config.options.boot_default   );
 	WRITE_TAG_STRING(&map_data, TAG_MESSAGE,    config.options.boot_message   );
+	WRITE_TAG_STRING(&map_data, TAG_MASTER_PASSWORD,
+												config.options.boot_masterpswd);
 	WRITE_TAG_DATA  (&map_data, TAG_PROMPT,     config.options.boot_prompt,  4);
 	WRITE_TAG_DATA  (&map_data, TAG_DELAY,      config.options.boot_delay,   4);
 	WRITE_TAG_DATA  (&map_data, TAG_TIMEOUT,    config.options.boot_timeout, 4);
@@ -485,48 +489,56 @@ create_map_data
 	}
 	for (record = config.records; record; record = record->next)
 	{
-		password   = record->password
-				   ? record->password
-				   : config.default_password;
+		password	= record->password
+					? record->password
+					: config.default_password;
+
+		/* don't write a zero lenth password */
 		if (password && (*password == '\0'))
 		{
 			password = NULL;
 		}
 
-		restricted = record->restricted
-				   ? record->restricted
-				   : config.default_restricted;
+		restricted	= record->restricted
+					? record->restricted
+					: config.default_restricted;
+
+		/* don't write restricted flag if FALSE */
 		if (restricted && (*restricted == FALSE))
 		{
 			restricted = NULL;
 		}
 
-		cmdline  = record->args ? record->args : config.default_cmdline;
-		if (record->append)
-		{
-			p = malloc(strlen(cmdline) + strlen(record->append) + 2);
-			if (p == NULL)
-			{
-				error_nomemory();
-			}
-			strcpy(p, cmdline);
-			if (*p) strcat(p, " ");
-			strcat(p, record->append);
-			cmdline = p;
-		}
+		cmdline		= record->cmdline
+					? record->cmdline
+					: config.default_cmdline;
+		
+		append		= record->append
+					? record->append
+					: config.default_append;
 
-		if ((kernel = record->kernel ? record->kernel : config.default_kernel)
-							== NULL)
+		kernel		= record->kernel
+					? record->kernel
+					: config.default_kernel;
+
+		console		= record->console
+					? record->console
+					: config.default_console;
+
+		root		= record->root
+					? record->root
+					: config.default_root;
+
+		read_only	= record->read_only
+					? record->read_only
+					: config.default_read_only;
+
+		/* must have a kernel */
+		if ((kernel == NULL) || (*kernel == '\0'))
 		{
 			die("No kernel image specified for `%s'\n", record->label);
 		}
 
-		if (restricted && (password == NULL))
-		{
-			die("Restricted requested without password for `%s'\n",
-				record->label);
-		}
-
 		/* find default boot record as we go */
 		if (equal_strings(config.options.boot_default, record->label)
 		||	equal_strings(config.options.boot_default, record->alias))
@@ -547,6 +559,10 @@ create_map_data
 		WRITE_TAG_STRING(&map_data, TAG_SYMTAB,          record->symtab       );
 		WRITE_TAG_DATA  (&map_data, TAG_MEMSIZE,         record->memsize,    4);
 		WRITE_TAG_DATA  (&map_data, TAG_CALLMONITOR,     record->callmonitor,4);
+		WRITE_TAG_STRING(&map_data, TAG_APPEND,          append               );
+		WRITE_TAG_STRING(&map_data, TAG_ROOT,            root                 );
+		WRITE_TAG_STRING(&map_data, TAG_CONSOLE,         console              );
+		WRITE_TAG_DATA  (&map_data, TAG_READ_ONLY,       read_only,          4);
 		write_tag_data  (&map_data, TAG_BOOT_RECORD_END, NULL,               0);
 	}
 
@@ -601,7 +617,7 @@ create_map_data
 }
 
 /*--------------------------------------------------------------------------*/
-/* Write the Loader (incl. Header Code)
+/* Write the Loader
  */
 
 static
@@ -987,6 +1003,10 @@ parse_opts
 
         config.boot_device_name = device;
     }
+    else if (config.boot_device_name == NULL)
+   	{
+   		die("No device given on command line or config file\n");
+    }
 
 	return;
 
@@ -1032,9 +1052,9 @@ main
 		die("%s is not a block special device\n", config.boot_device_name);
 	}
 
-	if (is_scsi_disk(info.st_rdev))
+	if (is_scsi_partition(info.st_rdev))
 	{
-		die("%s is a hard disk partition\n", config.boot_device_name);
+		die("%s is a SCSI disk partition\n", config.boot_device_name);
 	}
 
     boot_device = info.st_rdev;
